﻿using System;
using System.Data;
using System.Globalization;
using System.Reflection;
using System.Text;
using System.Transactions;
using Produmex.Foundation;
using Produmex.Foundation.Data;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Providers;

// PmxWorkflowExecutionTypes.SUB_FLOW
namespace WorkflowScript_LogOnScript_KF
{
    public class WorkflowScript_LogOnScript_KF : WorkflowInstanceScriptBase
    {
        // Input parameters
        // <none>

        // Output parameters
        public ReadWriteBinder<CultureInfo> DefaultCultureInfo;
        public ReadWriteBinder<PmxOseCompany> PmxOseCompany;
        public ReadWriteBinder<bool> LogOnCanceled;

        // Sub-flows
        private WorkflowInstanceBase m_getInitialData;
        private CultureInfo m_getInitialData_DefaultCultureInfo
        {
            get { return DefaultCultureInfo.Get(); }
            set { DefaultCultureInfo.Set(value); }
        }
        private PmxOseCompany m_getInitialData_PmxOseCompany
        {
            get { return PmxOseCompany.Get(); }
            set { PmxOseCompany.Set(value); }
        }

        public WorkflowScript_LogOnScript_KF(WorkflowInstanceBase parent, WorkflowInstanceFactory factory) : base(parent, factory)
        {
            // Initialize required sub-flows
            m_getInitialData = Create("GetInitialDataScript");
            m_getInitialData.BindParameters(
                new ReadWriteBinder<CultureInfo>("DefaultCultureInfo", this, "m_getInitialData_DefaultCultureInfo"),
                new ReadWriteBinder<PmxOseCompany>("PmxOseCompany", this, "m_getInitialData_PmxOseCompany")
            );
        }

        private static bool s_loggedOn = false;

        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);


        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {
            // Parameters in scope
            Session session = GetScopeParameter("Session") as Session;

            // Init our output parameters
            LogOnCanceled.Set(false);

            Message msg;
            ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;


            PmxOseCompany pmxOseCompany = sboProviderService.InvokeMethodWithDbConnection<PmxOseCompany>(false, true, null, null, new DelegateWithPmxDbConnection<PmxOseCompany>(ExecuteGetCompany));

            if (pmxOseCompany != null)
            {
                PmxOseCompany.Set(pmxOseCompany);
            }
            else
            {
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "InitialErrorKey", null,
                            "MessageKey", KnownMessageKeys.MSG_ERROR_NO_COMPANY_FOUND,
                            "ShowButton", true));

                WaitForMessage();
            }

            string userName = null;

        Step_10:

            sboProviderService.LogOffWithSessionId(session.Id);

            s_loggedOn = false;

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ILogonScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                    "Session", session,
                    "DefaultUserName", userName,
                    "ImageLocation", PmxOseCompany.Get().CompanyLogoLocation));
            msg = WaitForMessage();

            if (msg.Name == "Produmex.Foundation.SlimScreen.Interfaces.ILogonScreenReply.Canceled")
            {
                LogOnCanceled.Set(true);
                return;
            }
            if (msg.Name != "Produmex.Foundation.SlimScreen.Interfaces.ILogonScreenReply.CredentialsEntered")
            {
                goto Step_10;
            }

            userName = ExtractParameter<string>(msg.Parameters, "UserName");

            DateTime dt = DateTime.Now;
           
            ProdumexError pmxError = sboProviderService.LogOnWithSessionId(
                session.Id,
                userName,
                ExtractParameter<string>(msg.Parameters, "Password"));

            s_loggedOn = true;

            if (pmxError != null)
            {
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                    DefaultCultureInfo.Get(),
                    BuildParamCollection(
                        "PmxError", pmxError,
                        "ShowButton", true));
                WaitForMessage();

                goto Step_10;
            }

            string query = GetLogLoginQuery(
                SqlCommandHelper.FormatAndEscapeColumnValue(dt.Date, CultureInfo.CurrentCulture),
                dt.Hour * 100 + dt.Minute,
                System.Security.Principal.WindowsIdentity.GetCurrent().Name,
                userName);
            sboProviderService.InvokeMethodWithDbConnection<object>(false, true, null, null,
                delegate (PmxDbConnection conn, object[] parameters)
                {
                    LogLogin(query, conn);
                    return null;
                });

            // Execute sub-flow with logged on user
            base.StartSubFlowAndWait(m_getInitialData, true);

            sboProviderService.InvokeMethodWithDbConnection<bool>(false, null, null,
                new DelegateWithSboConnection<bool>(ExecuteLogon),
                userName);
        }

        #endregion

        private PmxOseCompany ExecuteGetCompany(PmxDbConnection conn, object[] parameters)
        {
            PmxOseCompanyProvider provCompany = new PmxOseCompanyProvider(conn);
            PmxOseCompany company = provCompany.GetCompany();


            if (company != null)
            {

                StringBuilder query = new StringBuilder();
                query.AppendFormat("SELECT * FROM {0} WHERE {1} = {2}",
                    SqlCommandHelper.EscapeTableName(SboUserLanguageTable.NAME),
                    SqlCommandHelper.EscapeTableColumnName(SboUserLanguageTable.NAME, SboUserLanguageTable.Columns.Code.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(company.LanguageCode, conn.DbCultureInfo));

                using (ISboRecordset rs = SboRecordsetHelper.RunQuery(s_log, query.ToString(), conn))
                {
                    if (!rs.EoF)
                    {
                        DefaultCultureInfo.Set(new CultureInfo(rs.GetTypedValue<string>(PmxUserLanguageTable.Columns.CultureName.NAME)));
                    }
                }
            }

            return company;
        }

        private bool ExecuteLogon(SboConnection conn, object[] parameters)
        {
            string userName = (string)parameters[0];

            Produmex.Sbo.Logex.Data.Providers.PmxUserProvider userProv = new Produmex.Sbo.Logex.Data.Providers.PmxUserProvider(conn);
            int? userLanguageCode = userProv.GetUserLanguageCode(userName);

            Produmex.Foundation.Data.Sbo.Providers.PmxUserLanguageProvider prov = new Produmex.Foundation.Data.Sbo.Providers.PmxUserLanguageProvider(conn);

            if (userLanguageCode.HasValue)
            {
                DefaultCultureInfo.Set(prov.GetCultureInfo(userLanguageCode.Value));
            }
            else
            {
                DefaultCultureInfo.Set(prov.GetCultureInfo(m_getInitialData_PmxOseCompany.LanguageCode));
            }

            return true;
        }

        private void LogLogin(string query, PmxDbConnection conn)
        {
            using (TransactionScope scope = PmxDbConnection.GetNewTransactionScope())
            {
                SboRecordsetHelper.RunQueryNoReturn(s_log, query, conn);
                scope.Complete();
            }
        }

        private string GetLogLoginQuery(string date, int time, string winName, string userName)
        {
            StringBuilder qb = new StringBuilder();
            StringBuilder subquery = new StringBuilder();

            subquery.AppendFormat("SELECT ISNULL(MAX( CAST( CODE AS INT )), 0) FROM {0}",
            SqlCommandHelper.EscapeTableName("@PMX_KF_LOGIN"));

            qb.AppendFormat("INSERT INTO {0} VALUES(({1})+1, CAST(({1})+1 AS NVARCHAR(254)), {2}, {3}, '{4}', '{5}')",
                SqlCommandHelper.EscapeTableName("@PMX_KF_LOGIN"),
                subquery,
                date,
                time,
                winName,
                userName);

            return qb.ToString();
        }
    }
}
